@@ -9,13 +9,16 @@ from typing import (Any,
                     Iterable,
                     List,
                     Sequence,
-                    Type)
+                    Tuple,
+                    Type,
+                    TypeVar)
 
 from clipping.hints import (Base,
                             BoundingBox,
                             Contour,
                             Coordinate,
                             Multipolygon,
+                            Multisegment,
                             Point,
                             Polygon,
                             Segment)
@@ -26,39 +29,65 @@ def all_equal(iterable: Iterable[Any]) -> bool:
     return next(groups, True) and not next(groups, False)
 
 
+Domain = TypeVar('Domain')
+
+
+def sort_pair(pair: Tuple[Domain, Domain]) -> Tuple[Domain, Domain]:
+    first, second = pair
+    return (first, second) if first < second else (second, first)
+
+
 def to_multipolygon_contours(multipolygon: Multipolygon) -> Iterable[Contour]:
     for border, holes in multipolygon:
         yield border
         yield from holes
 
 
-def to_bounding_box(multipolygon: Multipolygon) -> BoundingBox:
-    ((first_vertex, *rest_vertices), _), *rest_polygons = multipolygon
-    x_min, y_min = x_max, y_max = first_vertex
-    for x, y in _flatten([rest_vertices]
-                         + [border for border, _ in rest_polygons]):
+def to_bounding_box(points: Iterable[Point]) -> BoundingBox:
+    points = iter(points)
+    x_min, y_min = x_max, y_max = next(points)
+    for x, y in points:
         x_min, x_max = min(x_min, x), max(x_max, x)
         y_min, y_max = min(y_min, y), max(y_max, y)
     return x_min, x_max, y_min, y_max
 
 
+def are_bounding_boxes_disjoint(left: BoundingBox, right: BoundingBox) -> bool:
+    left_x_min, left_x_max, left_y_min, left_y_max = left
+    right_x_min, right_x_max, right_y_min, right_y_max = right
+    return (left_x_min > right_x_max or left_x_max < right_x_min
+            or left_y_min > right_y_max or left_y_max < right_y_min)
+
+
 def to_segments(contour: Contour) -> List[Segment]:
     return [(contour[index], contour[(index + 1) % len(contour)])
             for index in range(len(contour))]
 
 
+def to_mixed_base(multisegment: Multisegment,
+                  multipolygon: Multipolygon) -> Base:
+    return max(to_multisegment_base(multisegment),
+               to_multipolygon_base(multipolygon),
+               key=_bases_key)
+
+
 def to_multipolygon_base(multipolygon: Multipolygon) -> Base:
     return max({to_contour_base(contour)
                 for contour in to_multipolygon_contours(multipolygon)},
                key=_bases_key)
 
 
-def to_contour_base(contour: Contour) -> Base:
-    return max(set(map(type, _flatten(contour))),
+def to_multisegment_base(multisegment: Multisegment) -> Base:
+    return max(set(map(type, flatten(flatten(multisegment)))),
                key=_bases_key)
 
 
-_flatten = chain.from_iterable
+def to_contour_base(contour: Contour) -> Base:
+    return max(set(map(type, flatten(contour))),
+               key=_bases_key)
+
+
+flatten = chain.from_iterable
 
 
 def _bases_key(base: Base,
@@ -83,6 +112,11 @@ def to_rational_multipolygon(multipolygon: Multipolygon) -> Multipolygon:
             for border, holes in multipolygon]
 
 
+def to_rational_multisegment(multisegment: Multisegment) -> Multisegment:
+    return [(to_rational_point(start), to_rational_point(end))
+            for start, end in multisegment]
+
+
 def to_rational_contour(contour: Contour) -> Contour:
     return [to_rational_point(vertex) for vertex in contour]
 
@@ -99,3 +133,7 @@ def to_first_boundary_vertex(polygon: Polygon) -> Point:
 
 def to_multipolygon_x_max(multipolygon: Multipolygon) -> Coordinate:
     return max(x for border, _ in multipolygon for x, _ in border)
+
+
+def to_multisegment_x_max(multisegment: Multisegment) -> Coordinate:
+    return max(x for x, _ in flatten(multisegment))
