@@ -13,4 +13,4 @@ Polygon = Tuple[Contour, List[Contour]]
 Multipoint = List[Point]
 Multipolygon = List[Polygon]
 Multisegment = List[Segment]
-GeometryCollection = Tuple[Multipoint, Multisegment, Multipolygon]
+Mix = Tuple[Multipoint, Multisegment, Multipolygon]
